==> react-native-hotqa-sdk-0.0.22.tgz: package/src/components/audio-recording/index.js <==
import React, {Component} from 'react';
import {
  Button,
  PermissionsAndroid,
  Platform,
  SafeAreaView,
  StyleSheet,
  Alert,
  Text,
  View,
  TouchableOpacity,
} from 'react-native';
import Slider from '@react-native-community/slider';
import {
  Player,
  Recorder,
  MediaStates,
} from '@react-native-community/audio-toolkit';
import Feather from 'react-native-vector-icons/Feather';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import {connect} from 'react-redux';
import {REQUEST_SUBMIT_USES_ANSWER} from '../../redux/actions';

class AudioRecording extends Component {
  constructor(props) {
    super(props);
    this.state = {
      playPauseButton: 'Preparing...',
      recordButton: 'Record',

      stopButtonDisabled: true,
      playButtonDisabled: false,
      recordButtonDisabled: true,

      loopButtonStatus: false,
      progress: 0,

      error: null,
    };
  }

  componentWillMount() {
    this.player = null;
    this.recorder = null;
    this.lastSeek = 0;

    this._progressInterval = setInterval(() => {
      if (this.player && this._shouldUpdateProgressBar()) {
        let currentProgress =
          Math.max(0, this.player.currentTime) / this.player.duration;
        if (isNaN(currentProgress)) {
          currentProgress = 0;
        }
        this.setState({progress: currentProgress});
      }
    }, 100);
  }

  componentDidMount() {
    console.log(85, this.props.submitData);
  }

  componentWillUnmount() {
    clearInterval(this._progressInterval);
  }

  _shouldUpdateProgressBar() {
    // Debounce progress bar update by 200 ms
    return Date.now() - this.lastSeek > 200;
  }

  filename = () => {
    const {item} = this.props;
    return `record_${item.test_id}_${item.id}.mp4`;
  };

  _updateState() {
    this.setState({
      playPauseButton: this.player && this.player.isPlaying ? 'Pause' : 'Play',
      recordButton:
        this.recorder && this.recorder.isRecording ? 'Stop' : 'Record',

      stopButtonDisabled: !this.player || !this.player.canStop,
      playButtonDisabled:
        !this.player || !this.player.canPlay || this.recorder.isRecording,
      recordButtonDisabled:
        !this.recorder || (this.player && !this.player.isStopped),
    });
  }

  _playPause() {
    const user_answer = this.props.submitData;
    const {item} = this.props;
    const answerExisted = user_answer.find(
      (answer) => answer.question_id === item.id,
    );
    if (answerExisted) {
      this.player = answerExisted.player;
      this.player.playPause((err, paused) => {
        console.log('tienvt 81', err, paused);
        this._updateState();
      });
    } else {
      Alert.alert('Bạn hãy ghi âm lại câu trả lời');
    }
  }

  _stop() {
    if (this.player) {
      this.player.stop((err) => {
        this._updateState();
      });
    }
  }

  _seek(percentage) {
    if (!this.player) {
      return;
    }

    this.lastSeek = Date.now();

    let position = percentage * this.player.duration;

    this.player.seek(position, () => {
      this._updateState();
    });
  }

  _reloadPlayer() {
    // if (this.player) {
    //   this.player.destroy();
    // }
    this.player = new Player(this.filename(), {
      autoDestroy: false,
    }).prepare((err) => {
      if (err) {
        console.log('error at _reloadPlayer():');
        console.log(err);
      } else {
        this.player.looping = this.state.loopButtonStatus;
      }

      this._updateState();
    });

    this._updateState();

    this.player.on('ended', () => {
      this._updateState();
    });
    this.player.on('pause', () => {
      this._updateState();
    });

    const {item, dispatch} = this.props;
    // console.log(145, item);
    const user_choose = {value: ''};
    const data_answer = {
      player: this.player,
      record: this.recorder,
      section_id: item.section_id,
      section_tag: item.section_tag,
      question_id: item.id,
      question_configuration: JSON.parse(item.configurations),
      user_answer: user_choose,
      type: item.type,
    };
    dispatch({
      type: REQUEST_SUBMIT_USES_ANSWER,
      payload: {data: data_answer},
    });
  }

  _reloadRecorder() {
    if (this.state.recordButton === 'Stop') {
      this.props.onPauseAudio('Stop');
      this.recorder.stop(() => {
        this._reloadPlayer();
        this.setState({playButtonDisabled: false});
      });
    } else {
      this.props.onPauseAudio('Record');
      if (this.recorder) {
        this.recorder.destroy(() => {
          this.newRecord();
        });
      } else {
        this.newRecord();
      }
    }
  }

  newRecord = () => {
    this.recorder = new Recorder(this.filename(), {
      bitrate: 256000,
      channels: 2,
      sampleRate: 44100,
      format: 'mp4',
      quality: 'medium',
    }).record((err) => {
      console.log(204, err);
    });
    this.setState({recordButton: 'Stop', playButtonDisabled: true});
  };

  _toggleRecord() {
    if (this.player) {
      this.player.destroy();
    }

    let recordAudioRequest;
    if (Platform.OS == 'android') {
      recordAudioRequest = this._requestRecordAudioPermission();
    } else {
      recordAudioRequest = new Promise(function (resolve, reject) {
        resolve(true);
      });
    }

    recordAudioRequest.then((hasPermission) => {
      if (!hasPermission) {
        this.setState({
          error: 'Record Audio Permission was denied',
        });
        return;
      }
      this._reloadRecorder();
      //   this.recorder.toggleRecord((err, stopped) => {
      //     if (err) {
      //       this.setState({
      //         error: err.message,
      //       });
      //     }
      //     if (stopped) {
      //   // this._reloadPlayer();
      //     }
      //   this._updateState();
      //   });
    });
  }

  async _requestRecordAudioPermission() {
    try {
      const granted = await PermissionsAndroid.request(
        PermissionsAndroid.PERMISSIONS.RECORD_AUDIO,
        {
          title: 'Microphone Permission',
          message:
            'ExampleApp needs access to your microphone to test react-native-audio-toolkit.',
          buttonNeutral: 'Ask Me Later',
          buttonNegative: 'Cancel',
          buttonPositive: 'OK',
        },
      );
      if (granted === PermissionsAndroid.RESULTS.GRANTED) {
        return true;
      } else {
        return false;
      }
    } catch (err) {
      console.error(err);
      return false;
    }
  }

  _toggleLooping(value) {
    this.setState({
      loopButtonStatus: value,
    });
    if (this.player) {
      this.player.looping = value;
    }
  }

  render() {
    return (
      <View style={styles.content}>
        <Text style={styles.title}>Recording</Text>
        {this.state.recordButton === 'Record' ? (
          <TouchableOpacity
            onPress={() => this._toggleRecord()}
            style={styles.buttonRecord}>
            <Feather name="mic" size={50} color="#fff" />
          </TouchableOpacity>
        ) : (
          <TouchableOpacity
            onPress={() => this._toggleRecord()}
            style={styles.buttonRecord}>
            <FontAwesome name="square" size={30} color="#fff" />
          </TouchableOpacity>
        )}
        <View
          style={{
            flexDirection: 'row',
            width: '100%',
            justifyContent: 'center',
          }}>
          {this.state.playButtonDisabled ? (
            <Button title="Stop" onPress={() => this._stop()} />
          ) : (
            <Button title="Play" onPress={() => this._playPause()} />
          )}
          <Slider
            step={0.0001}
            style={{width: '80%'}}
            disabled={this.state.playButtonDisabled}
            onValueChange={(percentage) => this._seek(percentage)}
            value={this.state.progress}
            minimumTrackTintColor="#3287FB"
            maximumTrackTintColor="#B7B8B8"
          />
        </View>
      </View>
    );
  }
}

function mapStateToProps(state) {
  return {
    submitData: state.submitData.user_answer,
  };
}
export default connect(mapStateToProps)(AudioRecording);

const styles = StyleSheet.create({
  content: {
    marginTop: '5%',
    backgroundColor: '#718ac9',
    width: '100%',
    height: 250,
    alignItems: 'center',
    justifyContent: 'center',
  },
  buttonRecord: {
    backgroundColor: 'tomato',
    width: 100,
    height: 100,
    borderRadius: 50,
    alignItems: 'center',
    justifyContent: 'center',
  },
  slider: {
    height: 10,
    margin: 10,
    marginBottom: 50,
  },
  settingsContainer: {
    alignItems: 'center',
  },
  container: {
    borderRadius: 4,
    borderWidth: 0.5,
    borderColor: '#d6d7da',
  },
  title: {
    fontSize: 19,
    fontWeight: 'bold',
    textAlign: 'center',
    padding: 20,
  },
  errorMessage: {
    fontSize: 15,
    textAlign: 'center',
    padding: 10,
    color: 'red',
  },
});

==> react-native-hotqa-sdk-0.1.1.tgz: package/src/components/audio-recording/index.js <==
import React, {Component} from 'react';
import {
  PermissionsAndroid,
  Platform,
  Image,
  StyleSheet,
  Alert,
  Text,
  View,
  TouchableOpacity,
} from 'react-native';
import Slider from '@react-native-community/slider';
import {Player, Recorder} from '@react-native-community/audio-toolkit';
import Feather from 'react-native-vector-icons/Feather';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import {connect} from 'react-redux';
import {REQUEST_SUBMIT_USES_ANSWER} from '../../redux/actions';
import * as colors from '../../assets/colors';
import ic_play_record from '../../assets/images/ic_play_record.png';
import ic_play_disable from '../../assets/images/ic_play_disable.png';
import ic_pause_record from '../../assets/images/ic_pause_record.png';

class AudioRecording extends Component {
  constructor(props) {
    super(props);
    this.state = {
      playPauseButton: 'Play',
      recordButton: 'Record',

      // stopButtonDisabled: true,
      // playButtonDisabled: false,
      // recordButtonDisabled: true,
      isRecording: false,

      loopButtonStatus: false,
      progress: 0,

      error: null,
    };
  }

  componentWillMount() {
    this.player = null;
    this.recorder = null;
    this.lastSeek = 0;

    this._progressInterval = setInterval(() => {
      if (this.player && this._shouldUpdateProgressBar()) {
        let currentProgress =
          Math.max(0, this.player.currentTime) / this.player.duration;
        if (isNaN(currentProgress)) {
          currentProgress = 0;
        }
        this.setState({progress: currentProgress});
      }
    }, 100);
  }

  componentDidMount() {
    // console.log(85, this.props.submitData);
  }

  componentWillUnmount() {
    clearInterval(this._progressInterval);
  }

  _shouldUpdateProgressBar() {
    // Debounce progress bar update by 200 ms
    return Date.now() - this.lastSeek > 200;
  }

  filename = () => {
    const {item} = this.props;
    return `record_${item.test_id}_${item.id}.mp4`;
  };

  _updateState() {
    this.setState({
      playPauseButton: this.player && this.player.isPlaying ? 'Pause' : 'Play',
      recordButton:
        this.recorder && this.recorder.isRecording ? 'Stop' : 'Record',

      // stopButtonDisabled: !this.player || !this.player.canStop,
      // playButtonDisabled:
      //   !this.player || !this.player.canPlay || this.recorder.isRecording,
      // recordButtonDisabled:
      //   !this.recorder || (this.player && !this.player.isStopped),
    });
  }

  isAnswerExisted = () => {
    const user_answer = this.props.submitData;
    const {item} = this.props;
    const answerExisted = user_answer.find(
      (answer) => answer.question_id === item.id,
    );
    if (answerExisted) {
      this.player = answerExisted.player;
      return true;
    }
    return false;
  };

  _playPause() {
    // const user_answer = this.props.submitData;
    // const {item} = this.props;
    // const answerExisted = user_answer.find(
    //   (answer) => answer.question_id === item.id,
    // );
    if (this.isAnswerExisted()) {
      // this.player = answerExisted.player;
      this.player.playPause((err, paused) => {
        console.log('tienvt 81', err, paused);
        // this._updateState();
        this.setState({playPauseButton: 'Pause'});
      });
    } else {
      Alert.alert('Bạn chưa ghi âm câu trả lời');
    }
  }

  _stop() {
    if (this.player) {
      this.player.stop(() => {
        // this._updateState();
        this.setState({playPauseButton: 'Play'});
      });
    }
  }

  _seek(percentage) {
    if (!this.player) {
      return;
    }

    this.lastSeek = Date.now();

    let position = percentage * this.player.duration;

    this.player.seek(position, () => {
      this._updateState();
    });
  }

  _reloadPlayer() {
    // if (this.player) {
    //   this.player.destroy();
    // }
    this.player = new Player(this.filename(), {
      autoDestroy: false,
    }).prepare((err) => {
      if (err) {
        console.log('error at _reloadPlayer():');
        console.log(err);
      } else {
        this.player.looping = this.state.loopButtonStatus;
      }

      this._updateState();
    });

    this._updateState();

    this.player.on('ended', () => {
      this._updateState();
    });
    this.player.on('pause', () => {
      this._updateState();
    });

    const {item, dispatch} = this.props;
    // console.log(145, item);
    const user_choose = {value: ''};
    const data_answer = {
      player: this.player,
      record: this.recorder,
      section_id: item.section_id,
      section_tag: item.section_tag,
      question_id: item.id,
      question_configuration: JSON.parse(item.configurations),
      user_answer: user_choose,
      type: item.type,
    };
    dispatch({
      type: REQUEST_SUBMIT_USES_ANSWER,
      payload: {data: data_answer},
    });
  }

  _reloadRecorder() {
    if (this.state.recordButton === 'Stop') {
      this.props.onPauseAudio('Stop');
      this.recorder.stop(() => {
        this._reloadPlayer();
        // this.setState({recordButton: 'Record'});
      });
    } else {
      this.props.onPauseAudio('Record');
      if (this.recorder) {
        this.recorder.destroy(() => {
          this.newRecord();
        });
      } else {
        this.newRecord();
      }
    }
  }

  newRecord = () => {
    this.recorder = new Recorder(this.filename(), {
      bitrate: 256000,
      channels: 2,
      sampleRate: 44100,
      format: 'mp4',
      quality: 'medium',
    }).record((err) => {
      console.log(204, err);
    });
    this.setState({recordButton: 'Stop'});
  };

  _toggleRecord() {
    if (this.player) {
      this.player.destroy();
    }

    let recordAudioRequest;
    if (Platform.OS === 'android') {
      recordAudioRequest = this._requestRecordAudioPermission();
    } else {
      recordAudioRequest = new Promise(function (resolve, reject) {
        resolve(true);
      });
    }

    recordAudioRequest.then((hasPermission) => {
      if (!hasPermission) {
        this.setState({
          error: 'Record Audio Permission was denied',
        });
        return;
      }
      this._reloadRecorder();
      //   this.recorder.toggleRecord((err, stopped) => {
      //     if (err) {
      //       this.setState({
      //         error: err.message,
      //       });
      //     }
      //     if (stopped) {
      //   // this._reloadPlayer();
      //     }
      //   this._updateState();
      //   });
    });
  }

  async _requestRecordAudioPermission() {
    try {
      const granted = await PermissionsAndroid.request(
        PermissionsAndroid.PERMISSIONS.RECORD_AUDIO,
        {
          title: 'Microphone Permission',
          message:
            'ExampleApp needs access to your microphone to test react-native-audio-toolkit.',
          buttonNeutral: 'Ask Me Later',
          buttonNegative: 'Cancel',
          buttonPositive: 'OK',
        },
      );
      if (granted === PermissionsAndroid.RESULTS.GRANTED) {
        return true;
      } else {
        return false;
      }
    } catch (err) {
      console.error(err);
      return false;
    }
  }

  _toggleLooping(value) {
    this.setState({
      loopButtonStatus: value,
    });
    if (this.player) {
      this.player.looping = value;
    }
  }

  disablePlay = () => {
    if (this.state.recordButton === 'Stop') {
      return true;
    } else {
      if (!this.isAnswerExisted()) {
        return true;
      } else {
        return false;
      }
    }
  };

  render() {
    return (
      <View style={styles.content}>
        <View>
          {this.state.recordButton === 'Record' ? (
            <TouchableOpacity
              disabled={this.props.isFinish}
              onPress={() => this._toggleRecord()}
              style={styles.buttonRecord}>
              <Feather name="mic" size={35} color="#fff" />
            </TouchableOpacity>
          ) : (
            <TouchableOpacity
              onPress={() => this._toggleRecord()}
              style={styles.buttonRecord}>
              <FontAwesome name="square" size={25} color="#fff" />
            </TouchableOpacity>
          )}
          <Text style={styles.title}>Record</Text>
        </View>
        <View style={styles.viewSlide}>
          <Slider
            step={0.0001}
            disabled={this.disablePlay()}
            onValueChange={(percentage) => this._seek(percentage)}
            value={this.state.progress}
            minimumTrackTintColor={colors.color_orange}
            maximumTrackTintColor="#FFFFFF"
          />
          {this.state.playPauseButton === 'Pause' ? (
            <TouchableOpacity
              onPress={() => this._stop()}
              style={styles.btnPlay}>
              <Image
                source={ic_pause_record}
                resizeMode="contain"
                style={styles.btnControl}
              />
            </TouchableOpacity>
          ) : (
            <TouchableOpacity
              onPress={() => this._playPause()}
              disabled={this.disablePlay()}
              style={styles.btnPlay}>
              <Image
                source={!this.disablePlay() ? ic_play_record : ic_play_disable}
                resizeMode="contain"
                style={styles.btnControl}
              />
            </TouchableOpacity>
          )}
        </View>
      </View>
    );
  }
}

function mapStateToProps(state) {
  return {
    submitData: state.submitData.user_answer,
  };
}
export default connect(mapStateToProps)(AudioRecording);

const styles = StyleSheet.create({
  content: {
    flexDirection: 'row',
    marginTop: 20,
    backgroundColor: colors.color_blue,
    alignItems: 'center',
    marginEnd: '10%',
    borderRadius: 20,
    padding: 20,
  },
  buttonRecord: {
    backgroundColor: colors.color_orange_4,
    width: 70,
    height: 70,
    borderRadius: 35,
    alignItems: 'center',
    justifyContent: 'center',
  },
  settingsContainer: {
    alignItems: 'center',
  },
  container: {
    borderRadius: 4,
    borderWidth: 0.5,
    borderColor: '#d6d7da',
  },
  title: {
    fontSize: 16,
    textAlign: 'center',
    color: 'white',
    marginTop: 6,
  },
  errorMessage: {
    fontSize: 15,
    textAlign: 'center',
    padding: 10,
    color: 'red',
  },
  viewSlide: {
    flex: 1,
    marginStart: 20,
  },
  btnPlay: {
    alignSelf: 'center',
    // alignItems: 'center',
    // justifyContent: 'center',
  },
  btnControl: {
    width: 40,
    height: 40,
  },
});
